use v6;

# "module Foo; ..." is equivalent to "module Foo {...}", so it returns the Foo
# module object, see thread "What do use and require evaluate to?" on p6l
# started by Ingo Blechschmidt,
# L<"http://www.nntp.perl.org/group/perl.perl6.language/22258">.

module t::spec::packages::RequireAndUse3;

23;
