use v6;

# require and use return the last statement evaluated in the .pm, in this case
# 42. See thread "What do use and require evaluate to?" on p6l started by Ingo
# Blechschmidt, L<"http://www.nntp.perl.org/group/perl.perl6.language/22258">.

module t::spec::packages::RequireAndUse1 {
  23;
}

42;
